
#include "stdafx.h"
#include "ProIniFile.h"
#include "ProIniSection.h"



/////////////////////////////////////////////////////////////////////////////
// Methoden der Klasse CProIniFile
//
//

CProIniFile::CProIniFile()
{
	m_pfFile=NULL;
	m_strName="";
	m_posSection=NULL;
}


BOOL CProIniFile::Open(LPCTSTR lpszName)
{
	if((m_pfFile=fopen(lpszName,"rb"))==NULL) return FALSE;
	
	m_strName=lpszName;

	ReadAll();

	return TRUE;
}

// eine Zeile aus der Parameterfile lesen
// ein die Zeile abschlieendes 0x0d0a wird NICHT eingelesen
BOOL CProIniFile::ReadLineRaw(CString& str)
{
	char cbuf[400],c;
	int i=0;

	if(feof(m_pfFile)) return FALSE;

	memset((void *)cbuf,0,400);
	while((c=fgetc(m_pfFile))!=0x0a && !feof(m_pfFile))
	{
		if(i<400) {
			cbuf[i++]=c;
		}
	}
	// ein abschlieendes 0x0d eliminieren
	// (das 0x0a wird gar nicht erst in den Puffer geschrieben
	if(i>0)
	{
		if(cbuf[i-1]==0x0d) cbuf[i-1]=0;

		// rechtsseitig fhrende Spaces eliminieren
		i--;
		while(cbuf[i]==32 && i>0) cbuf[i--]=0;
	}

	if(cbuf[0]==';') {
		// beginnt diese Zeile mit einem Semikolon, so wird sie als Kommentat
		// interpretiert und ignoriert
		str="";
		return TRUE;
	}

	str=cbuf;
	return TRUE;
}

// eine Zeile aus der Parameterfile lesen
// ein die Zeile abschlieendes 0x0d0a wird NICHT eingelesen
// im Ggs. zu ReadLineRaw wrrden keine Kommentare und leere Zeilen eingelesen
BOOL CProIniFile::ReadLine(CString& str)
{
	char cbuf[400],c,cprev;
	int i=0;
	BOOL bisComment;

	if(feof(m_pfFile)) return FALSE;

	memset((void *)cbuf,0,400);
	
	// damit leere Zielen, bzw. Zeilen, die nur aus Kommentar
	// bestehen, nicht eingelesen werden, den Vorgang
	// des Einlesens solange wiederholen, bis eintweder eine 
	// gefllte Zeile eingelesen oder das Dateiende erreicht wurde
	while(i==0 && !feof(m_pfFile))
	{
		i=0;
		cprev=0;
		bisComment=FALSE;
		while((c=fgetc(m_pfFile))!=0x0a && !feof(m_pfFile))
		{
			// aus Tabs Spaces machen
			if(c==9) c=32;
			
			if(c==';') bisComment=TRUE;

			// aus mehrfachen Spaces einfache machen
			if(!bisComment && c!=0x0d && (c!=32 || (c==32 && cprev!=32)))
			{
				cbuf[i++]=c;
				cprev=c;
			}
		}
	}

	// ein abschlieendes 0x0d eliminieren
	// (das 0x0a wird gar nicht erst in den Puffer geschrieben
	if(i>0)
	{
		if(cbuf[i-1]==0x0d) cbuf[i-1]=0;

		// rechtsseitig fhrende Spaces eliminieren
		i--;
		while(cbuf[i]==32 && i>0) cbuf[i--]=0;
	}

	str=cbuf;
	return TRUE;
}

// testen, ob es sich bei einem String um die Kopfzeile 
// einer Section handelt
BOOL CProIniFile::IsSectionHeader(CString str) const
{
	// ein Sectionheader beginnt mit '['
	if(*((LPCSTR)str)=='[') return TRUE;
	else return FALSE;

}

// alle Sections der Parameterdatei in die Section-Liste
// m_listSection einlesen
void CProIniFile::ReadAll()
{
	CString strRaw="";
	CProIniSection* psec=NULL;

	// jeweils eine Zeile einelesen, bis EOF
	while(!feof(m_pfFile))
	{
		// ist diese Zeile ein Section-Header('[2]')
		if(IsSectionHeader(strRaw))
		{
			psec=new CProIniSection;
			// die Kopfzeile der Section hinzufgen
			psec->AddString(strRaw);
			// in diese Section so lange Zeilen einlesen,
			// bis entweder das Dateienede oder der nchste Section-Header kommt
			while(ReadLineRaw(strRaw) && !IsSectionHeader(strRaw))
			{
				// keine leeren Zeilen einlesen
				if(strRaw.GetLength()>0) psec->AddString(strRaw);
			}
		}
		else {
			ReadLineRaw(strRaw);
		}
		// wenn eine Section gelesen wurde, diese anhngen
		if(psec) {
			m_listSection.AddTail(psec);
			psec=NULL;
		}
	}

	fclose(m_pfFile);
	m_pfFile=NULL;
}

// eine Section 'raw' ermitteln
// der Zeiger wird so bergeben, wie die Section in der 
// internen Liste steht (also incl. Kommentare etc.)
// im Ggs. zu GetSection wird hier KEIN neues Objekt erzeugt, 
// sondern nur ein Zeiger auf ein Objekt der Liste bergeben
// es wird ein Zeiger aus das CProIniSection-Objekt zurckgegeben
CProIniSection* CProIniFile::GetSectionRaw(int inr) const
{
	CProIniSection* psec;
	if(inr>=GetCount()) return NULL;

	psec=(CProIniSection *)m_listSection.GetAt(m_listSection.FindIndex(inr));
	return psec;

}

// eine Section anhand ihres Namens ermitteln;
// es wird die erste Section dieses Namens ermittelt,
// wobei der Namensvergleich nicht case-sensitive ist
// die Section wird 'raw', d.h. unverndert zurckgegeben
CProIniSection* CProIniFile::GetSectionRaw(LPCTSTR lpszName) const
{
	CProIniSection* pSecFound=NULL;
	CString strTemp;
	CString strName=lpszName;
	strName.TrimRight();
	strName.TrimLeft();
	strName.MakeLower();
	for(POSITION pos=m_listSection.GetHeadPosition();pos!=NULL;) {
		CProIniSection* pSec=(CProIniSection*)m_listSection.GetNext(pos);
		strTemp=pSec->GetTitle();
		strTemp.MakeLower();
		if(strTemp==strName) {
			pSecFound=pSec;
			break;
		}
	}
	return pSecFound;
}

// eine vorher mit SetSectionPos bestimmte
// Section zurckgeben, und einen internen Zeiger
// um 1 weitersetzen
CProIniSection* CProIniFile::GetNextSectionRaw()
{
	if(m_posSection==NULL) return NULL;
	CProIniSection* psec;

	psec=(CProIniSection *)m_listSection.GetNext(m_posSection);
	return psec;
}

// einen Schritt zurckgehen
void CProIniFile::GoBack()
{
	if(m_posSection==NULL) {
		// es wurde ans Ende iteriert, dann wird m_posSection auf NULL gesetzt
		SetSectionPos(GetCount()-1);
	}
	else {
		m_listSection.GetPrev(m_posSection);
	}
}

// einen internen Zeiger auf eine bestimmte Section
// positionieren; 
// ein darfauffolgender Aufruf von GetNetSectionRaw liefert
// die besagt Section
void CProIniFile::SetSectionPos(int inr)
{
	if(inr<0 || inr>GetCount()-1) {
		m_posSection=NULL;
	}
	else {
		m_posSection=m_listSection.FindIndex(inr);
	}
}

// die Anzahl der Sections ermitteln
int CProIniFile::GetCount() const
{
	return m_listSection.GetCount();
}

// Parameterfile unter anderem Namen speichern
BOOL CProIniFile::SaveAs(CString strName)
{
	CProIniSection* psec;
	FILE *fp;
	int i,j,imax,jmax;
	imax=m_listSection.GetCount();
	CString str;
	CString strTemp;
	CString strTitle;

	m_strName=strName;

	if((fp=fopen(m_strName,"wb"))==NULL) return FALSE;

	// alle Sections durchgehen
	for(i=0;i<imax;i++)
	{
		psec=GetSectionRaw(i);
		jmax=psec->GetCount();

		// den Titel ermitteln
		if(psec->GetCount()!=0) {
			strTitle=psec->GetTitle();
		}
		else {
			strTitle=" ";
		}
		if(strTitle[0]!='[') {
			strTitle="["+strTitle+"]";
		}
		fputs(strTitle,fp);
		fputs("\15\12",fp);

		//
		// dann die brige Section
		//
		// in einer Section alle Strings durchgehen
		for(j=1;j<jmax;j++)
		{
			str=psec->GetString(j);

			// den String mit einem abschlieenden CRLF in die Datei schreiben
			fputs(str,fp);
			fputs("\15\12",fp);
		}
		fputs("\15\12",fp);
		fputs("\15\12",fp);
	}
	fclose(fp);
	return TRUE;
}


// eine Section umdefinieren
void CProIniFile::SetSection(CProIniSection* psec,int inr)
{
	CProIniSection* psecold;
	if(inr>=m_listSection.GetCount()) return;

	// das alte Element in der Liste lschen
	// (in dem Moment, wo der Verweis in der Liste berschrieben wird,
	// existiert kein Verweis mehr auf dieses Objekt)
	psecold=GetSectionRaw(inr);
	if(inr!=0)
		delete psecold;

	// den neuen Zeiger in der Liste eintragen
	m_listSection.SetAt(m_listSection.FindIndex(inr),psec);
}

// eine Section am Ende anfgen
void CProIniFile::AddSection(CProIniSection* psec)
{
	m_listSection.AddTail(psec);
}

CProIniFile::~CProIniFile()
{
	// alle Sections der Liste lschen
	CProIniSection *psec;
	while(!m_listSection.IsEmpty())
	{
		psec=(CProIniSection *)m_listSection.GetTail();
		m_listSection.RemoveTail();
		delete psec;
	}
	if(m_pfFile!=NULL) fclose(m_pfFile);
}

// eine Section aus der IniFile lschen
// der Index ist zero-based
void CProIniFile::DeleteAt(int inr)
{
	CProIniSection* psec;
	// wurde eine Section angewhlt, die nicht existiert ?
	if(inr>=m_listSection.GetCount()) return;
	psec=GetSectionRaw(inr);
	m_listSection.RemoveAt(m_listSection.FindIndex(inr));
	// diese Section lschen
	delete psec;
}

// an einer bestimmten Position eine Section einfgen
// es wird vor der angegebenen Position eingefgt
// (InsertBefore)
void CProIniFile::InsertSection(CProIniSection *psec, int iPos)
{
	m_listSection.InsertBefore(m_listSection.FindIndex(iPos),psec);
}


void CProIniFile::DeleteAll()
{
	// alle Sections der Liste lschen
	CProIniSection *psec;
	while(!m_listSection.IsEmpty())
	{
		psec=(CProIniSection *)m_listSection.GetTail();
		m_listSection.RemoveTail();
		delete psec;
	}
}

// eine Inidatei aus einem String einlesen
BOOL CProIniFile::FromString(LPCTSTR lpszText)
{
	LPCTSTR lpszRead=lpszText;

	CString strRaw="";
	CProIniSection* psec=NULL;

	// jeweils eine Zeile einelesen, bis EOF
	while(*lpszRead!=0)
	{
		// ist diese Zeile ein Section-Header('[2]')
		if(IsSectionHeader(strRaw))
		{
			psec=new CProIniSection;
			// die Kopfzeile der Section hinzufgen
			psec->AddString(strRaw);
			// in diese Section so lange Zeilen einlesen,
			// bis entweder das Dateienede oder der nchste Section-Header kommt
			while(ReadTextLineRaw(strRaw,lpszRead) && !IsSectionHeader(strRaw))
			{
				// keine leeren Zeilen einlesen
				if(strRaw.GetLength()>0) psec->AddString(strRaw);
			}
		}
		else {
			ReadTextLineRaw(strRaw,lpszRead);
		}
		// wenn eine Section gelesen wurde, diese anhngen
		if(psec) {
			m_listSection.AddTail(psec);
			psec=NULL;
		}
	}
	return TRUE;
}

// eine Zeile einer Inidatei aus einem Text einlesen
BOOL CProIniFile::ReadTextLineRaw(CString& str,LPCTSTR& lpszRead)
{
	char cbuf[400];
	char c;
	int i=0;

	if(*lpszRead==0) return FALSE;

	memset((void *)cbuf,0,400);
	while((c=*lpszRead++)!=0x0a && c!=0)
	{
		if(i<400) {
			cbuf[i++]=c;
		}
	}
	// ein abschlieendes 0x0d eliminieren
	// (das 0x0a wird gar nicht erst in den Puffer geschrieben
	if(i>0)
	{
		if(cbuf[i-1]==0x0d) cbuf[i-1]=0;

		// rechtsseitig fhrende Spaces eliminieren
		i--;
		while(cbuf[i]==32 && i>0) cbuf[i--]=0;
	}

	if(cbuf[0]==';') {
		// beginnt diese Zeile mit einem Semikolon, so wird sie als Kommentar
		// interpretiert und ignoriert
		str="";
		return TRUE;
	}

	str=cbuf;
	return TRUE;
}

